﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using TesterClasses;
using System.IO;

namespace StatsViewer
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        List<Profile> profiles = new List<Profile>();

        private int CompareByName(Profile x, Profile y)
        {
            return string.Compare(x.Name, y.Name);
        }

        private int CompareByGrade(Profile x, Profile y)
        {
            return string.Compare(x.Grade, y.Grade);
        }

        /*private int CompareByDate(Profile x, Profile y)
        {
            return DateTime.Compare(x, y);
        }*/

        private List<DateTime> GetAllDates()
        {
            List<DateTime> result = new List<DateTime>();
            foreach (Profile p in profiles)
            {
                foreach (TestResults tr in p)
                {
                    if (!result.Exists(delegate(DateTime d)
                                           {
                                               if (d.ToShortDateString() == tr.time.ToShortDateString())
                                                   return true;
                                               else
                                                   return false;
                                           }))
                    {
                        result.Add(tr.time);
                    }
                }
            }
            return result;
        }

        private void RefreshTree()
        {
            statsTextBox.Clear();
            treeView.Nodes.Clear();
            if (radioButton1.Checked)
            {
                profiles.Sort(new Comparison<Profile>(CompareByName));
                foreach (Profile p in profiles)
                {
                    TreeNode[] nodes = new TreeNode[p.Count];
                    for (int i = 0; i < p.Count; ++i)
                    {
                        nodes[i] = new TreeNode();
                        nodes[i].Text = p[i].testName + '(' + p[i].time.ToShortDateString() + ')';
                        nodes[i].Tag = p[i].ToString();
                    }
                    TreeNode main = new TreeNode(p.Name, nodes);
                    treeView.Nodes.Add(main);
                }
            }
            else if (radioButton2.Checked)
            {
                profiles.Sort(new Comparison<Profile>(CompareByGrade));
                string current = profiles[0].Grade;
                TreeNode node = new TreeNode(current);
                for (int i = 0; i < profiles.Count; ++i)
                {
                    if (profiles[i].Grade == current)
                    {
                        TreeNode[] children = new TreeNode[profiles[i].Count];
                        for (int j = 0; j < profiles[i].Count; ++j)
                        {
                            children[j] = new TreeNode();
                            children[j].Text = profiles[i][j].testName + '(' + profiles[i][j].time.ToShortDateString() + ')';
                            children[j].Tag = profiles[i][j].ToString();
                        }
                        TreeNode main = new TreeNode(profiles[i].Name, children);
                        node.Nodes.Add(main);
                    }
                    else
                    {
                        treeView.Nodes.Add(node);
                        node = null;
                        current = profiles[i].Grade;
                        node = new TreeNode(current);
                        --i;
                    }
                }
                treeView.Nodes.Add(node);
            }
            else if (radioButton3.Checked)
            {
                List<DateTime> dates = GetAllDates();
                foreach (DateTime d in dates)
                {
                    treeView.Nodes.Add(d.ToShortDateString(), d.ToShortDateString());
                }
                foreach (Profile p in profiles)
                {
                    foreach (TestResults tr in p)
                    {
                        TreeNode mainNode = treeView.Nodes[tr.time.ToShortDateString()];
                        TreeNode node = mainNode.Nodes[p.Name];
                        if (node == null)
                        {
                            //TreeNode tmp = new TreeNode(tr.testName + '(' + tr.time.ToShortDateString + ')');
                            //TreeNode mainTmp = new TreeNode(p.Name, new TreeNode[] { tmp });
                            mainNode.Nodes.Add(p.Name, p.Name);
                            TreeNode tmp = new TreeNode(tr.testName + '(' + tr.time.ToShortDateString() + ')');
                            tmp.Tag = tr.ToString();
                            mainNode.Nodes[mainNode.Nodes.Count - 1].Nodes.Add(tmp);
                        }
                        else
                        {
                            TreeNode tmp = new TreeNode(tr.testName + '(' + tr.time.ToShortDateString() + ')');
                            tmp.Tag = tr.ToString();
                            node.Nodes.Add(tmp);
                        }
                    }
                }
            }
        }

        private void ShowStats(TestResults results)
        {
            string[] resultString = {"Название теста: " + results.testName,
                "Правильных ответов: " + results.rightAnswers.ToString() + " из " + results.numberOfQuestions.ToString() + " (" + 
                ((double)results.rightAnswers / results.numberOfQuestions) * 100 + "%)",
                "Начало теста: " + results.time.ToString(),
                "Тест выполнен за " + results.interval.TotalMinutes + " минут"};
            statsTextBox.Lines = resultString;
        }

        private void ShowStats(TreeNode node)
        {
            double average = 0;
            if (radioButton1.Checked)
            {
                int count = node.Nodes.Count;
                foreach (TreeNode n in node.Nodes)
                {
                    TestResults tr = TestResults.Parse((string)n.Tag);
                    average += ((double)tr.rightAnswers / tr.numberOfQuestions * 100) / count;
                }
            }
            else
            {
                int count = 0;
                if (node.Nodes[0].Tag == null)
                {
                    foreach (TreeNode n in node.Nodes)
                        count += n.Nodes.Count;
                }
                foreach (TreeNode n in node.Nodes)
                {
                    if (n.Tag == null)
                    {
                        foreach (TreeNode nn in n.Nodes)
                        {
                            TestResults tr = TestResults.Parse((string)nn.Tag);
                            average += ((double)tr.rightAnswers / tr.numberOfQuestions * 100) / count;
                        }
                    }
                    else
                    {
                        int localCount = node.Nodes.Count;
                        TestResults tr = TestResults.Parse((string)n.Tag);
                        average += ((double)tr.rightAnswers / tr.numberOfQuestions * 100) / localCount;
                    }
                }
            }
            statsTextBox.Text = "Средний процент: " + average.ToString();
        }
        
        private void LoadProfile(string name, bool showWarning)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(name);
            if (doc["Profile"] == null)
            {
                if (showWarning)
                    MessageBox.Show(name + " не является профилем");
            }
            else
            {
                Profile p = new Profile(name);
                try
                {
                    p.LoadFromFile();
                }
                catch (Exception)
                {
                    MessageBox.Show(name + " не является корректным профилем");
                    return;
                }
                profiles.Add(p);
            }
        }

        private void addFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                //LoadProfile(openFileDialog1.FileName, true);
                string[] files = openFileDialog1.FileNames;
                foreach (string s in files)
                    LoadProfile(s, true);
            }
            RefreshTree();
        }

        private void addDirectoryToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                string[] files = Directory.GetFiles(folderBrowserDialog1.SelectedPath, "*.xml", SearchOption.TopDirectoryOnly);
                foreach (string s in files)
                    LoadProfile(s, false);
            }
            RefreshTree();
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node.Tag != null)
                ShowStats(TestResults.Parse((string)e.Node.Tag));
            else
                ShowStats(e.Node);
        }

        private void radioButton1_Click(object sender, EventArgs e)
        {
            RefreshTree();
        }

        private void treeView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete && treeView.SelectedNode != null && treeView.SelectedNode.Tag != null)
            {
                TestResults tr = TestResults.Parse((string)treeView.SelectedNode.Tag);
                foreach (Profile p in profiles)
                    p.Remove(tr);
                RefreshTree();
            }
        }
    }
}