﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TesterClasses;

namespace TestDesigner
{
    public partial class MainForm : Form
    {
        private string fileName = "";
        
        public MainForm()
        {
            InitializeComponent();
        }

        private void RefreshForm()
        {
            saveToolStripMenuItem.Enabled = true;
            TestNameTextBox.Enabled = true;
            panel1.Enabled = true;
            panel2.Enabled = false;
            DeleteQuestionButton.Enabled = false;
            Program.currentChoice = -1;
            Program.currentQuestion = -1;
            ChoiceList.Items.Clear();
            QuestionList.Items.Clear();
            TestNameTextBox.Text = Program.questions.TestName;
            QuestionTextBox.Text = "";
            ChoiceTextBox.Text = "";
            IsTrueCheckBox.Checked = false;
            for (int i = 1; i <= Program.questions.Count; ++i)
            {
                QuestionList.Items.Add("Вопрос #" + i.ToString());
            }
        }

        private void AddQuestionButton_Click(object sender, EventArgs e)
        {
            Program.questions.AddQuestion("");
            QuestionList.Items.Add("Вопрос #" + (QuestionList.Items.Count + 1));
            QuestionList.SelectedIndex = QuestionList.Items.Count - 1;
        }

        private void DeleteQuestionButton_Click(object sender, EventArgs e)
        {
            Program.questions.DeleteQuestion(Program.currentQuestion);
            //QuestionList.Items.RemoveAt(Program.currentQuestion);
            RefreshForm();
        }

        private void QuestionList_SelectedIndexChanged(object sender, EventArgs e)
        {
            ChoiceList.Items.Clear();
            if (QuestionList.SelectedIndex != -1)
            {
                panel2.Enabled = true;
                DeleteQuestionButton.Enabled = true;
                Program.currentQuestion = QuestionList.SelectedIndex;
                if (!QuestionTextBox.Enabled)
                    QuestionTextBox.Enabled = true;
                string[] delimiters = { @"\n" };
                string[] strings = Program.questions[Program.currentQuestion].Text.Split(delimiters, StringSplitOptions.RemoveEmptyEntries);
                QuestionTextBox.Lines = strings;
                for (int i = 1; i <= Program.questions[Program.currentQuestion].Count; ++i)
                    ChoiceList.Items.Add("Ответ #" + i.ToString());
                QuestionTextBox.Focus();
            }
            else
            {
                QuestionTextBox.Text = "";
                QuestionTextBox.Enabled = false;
            }
        }

        private void QuestionTextBox_TextChanged(object sender, EventArgs e)
        {
            //QuestionList.Items[Program.currentQuestion] = QuestionTextBox.Text;
            if (QuestionTextBox.Focused)
            {
                System.Text.StringBuilder builder = new StringBuilder();
                foreach (string s in QuestionTextBox.Lines)
                {
                    builder.Append(s + @"\n");
                }
                //builder.Remove(builder.Length - 2, 2);
                Program.questions[Program.currentQuestion].Text = builder.ToString();
            }
        }

        private void QuestionTextBox_Leave(object sender, EventArgs e)
        {
            //QuestionList.Items[Program.currentQuestion] = Program.questions[Program.currentQuestion].Text;
            //Program.questions[Program.currentQuestion].Text = QuestionTextBox.Text;

            //if (Program.questions[Program.currentQuestion].Text.Length > 0)
                //Program.questions[Program.currentQuestion].Text = Program.questions[Program.currentQuestion].Text.Remove(Program.questions[Program.currentQuestion].Text.Length - 1);
            if (Program.questions[Program.currentQuestion].Text.EndsWith("\n"))
                Program.questions[Program.currentQuestion].Text = Program.questions[Program.currentQuestion].Text.Remove(Program.questions[Program.currentQuestion].Text.Length - 2);
        }

        private void AddChoiceButton_Click(object sender, EventArgs e)
        {
            Program.questions[Program.currentQuestion].AddChoice("", false);
            ChoiceList.Items.Add("Ответ #" + (ChoiceList.Items.Count + 1));
            ChoiceList.SelectedIndex = ChoiceList.Items.Count - 1;
        }

        private void DeleteChoiceButton_Click(object sender, EventArgs e)
        {
            Program.questions[Program.currentQuestion].DeleteChoice(ChoiceList.SelectedIndex);
            ChoiceList.Items.RemoveAt(ChoiceList.SelectedIndex);
        }

        private void ChoiceTextBox_TextChanged(object sender, EventArgs e)
        {
            //ChoiceList.Items[Program.currentChoice] = ChoiceTextBox.Text;
            if (ChoiceTextBox.Focused)
                Program.questions[Program.currentQuestion][Program.currentChoice].Text = ChoiceTextBox.Text;
        }

        private void ChoiceList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ChoiceList.SelectedIndex != -1)
            {
                Program.currentChoice = ChoiceList.SelectedIndex;
                if (!ChoiceTextBox.Enabled)
                    ChoiceTextBox.Enabled = true;
                if (!IsTrueCheckBox.Enabled)
                    IsTrueCheckBox.Enabled = true;
                ChoiceTextBox.Text = Program.questions[Program.currentQuestion][Program.currentChoice].Text;
                IsTrueCheckBox.Checked = Program.questions[Program.currentQuestion][ChoiceList.SelectedIndex].IsTrue;
                ChoiceTextBox.Focus();
            }
            else
            {
                ChoiceTextBox.Text = "";
                ChoiceTextBox.Enabled = false;
                IsTrueCheckBox.Enabled = false;
            }
        }

        private void ChoiceTextBox_Leave(object sender, EventArgs e)
        {
            //ChoiceList.Items[Program.currentChoice] = ChoiceTextBox.Text;
            /*System.Text.StringBuilder builder = new StringBuilder();
            foreach (string s in ChoiceTextBox.Lines)
            {
                builder.Append(s + @"\n");
            }
            ChoiceList.Items[Program.currentChoice] = builder.ToString();*/
            //Program.questions[Program.currentQuestion][Program.currentChoice].Text = ChoiceTextBox.Text;
        }

        private void IsTrueCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            Program.questions[Program.currentQuestion][ChoiceList.SelectedIndex].IsTrue = IsTrueCheckBox.Checked;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (fileName != "")
                Program.questions.SaveToXml(fileName);
            else
                saveAsToolStripMenuItem_Click(sender, null);
        }

        private void loadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                Program.questions = new Questions();
                Program.questions.LoadFromXml(openFileDialog1.FileName);
                RefreshForm();
            }
        }

        private void newTestToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.questions = new Questions();
            RefreshForm();
        }

        private void TestNameTextBox_TextChanged(object sender, EventArgs e)
        {
            Program.questions.TestName = TestNameTextBox.Text;
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                fileName = saveFileDialog1.FileName;
                Program.questions.SaveToXml(saveFileDialog1.FileName);
            }
        }
    }
}