﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Xml;

namespace TesterClasses
{
    public class Question : IEnumerable, IEnumerator
    {
        private string question;
        private List<Choice> _choices = new List<Choice>();
        private int _numberOfAnswers = -1;
        
        public Question()
        {
            this._choices.Add(new Choice("Новый вариант", false));
        }

        public Question(string question)
        {
            this._choices.Add(new Choice("Новый вариант", false));
        }

        //Не создает в массиве объект. Нужно при загрузке из файла.
        //Второй параметр нужен только для того, чтобы отличить этот конструктор от предыдущего.
        internal Question(string question, int i)
        {
            this.question = question;
        }

        public void AddChoice(string text, bool isTrue)
        {
            Choice choice = new Choice(text, isTrue);
            _choices.Add(choice);
        }

        public void DeleteChoice(int number)
        {
            _choices.RemoveAt(number);
        }

        //Функция должна возвращать тип Choice, а не object.
        //Решение - использовать List<Choice>
        public Choice this[int pos]                 
        {                                            
            get { return _choices[pos]; }  
            set { _choices[pos] = value; }
        }

        public int Count
        {
            get { return _choices.Count; }
        }

        public string Text
        {
            get { return question; }
            set { question = value; }
        }

        public int NumberOfAnswers
        {
            get
            {
                if (_numberOfAnswers == -1)
                {
                    _numberOfAnswers = 0;
                    foreach (Choice c in _choices)
                    {
                        if (c.IsTrue)
                            ++_numberOfAnswers;
                    }
                }
                return _numberOfAnswers;
            }
        }

        public bool isTrue(int number)
        {
            if (_choices[number].IsTrue)
                return true;
            else
                return false;
        }

        internal void SaveToXml(XmlTextWriter writer)
        {
            writer.WriteStartElement("Question");
            writer.WriteAttributeString("text", question);
            foreach (Choice q in _choices)
            {
                q.SaveToXml(writer);
            }
            writer.WriteEndElement();
        }

        internal void LoadFromXml(XmlTextReader reader)
        {
            while (reader.NodeType != XmlNodeType.Element)
                reader.Read();
            this.Text = reader.GetAttribute(0);
            reader.Read();
            while (reader.Name != "Question")
            {
                AddChoice("NewChoice", false);
                _choices[_choices.Count - 1].LoadFromXml(reader);
            }
        }

        //IEnumerator and IEnumerable

        private int position = -1;

        public IEnumerator GetEnumerator()
        {
            return (IEnumerator)this;
        }

        public object Current
        {
            get { return _choices[position]; }
        }

        public bool MoveNext()
        {
            if (position < _choices.Count - 1)
            {
                position++;
                return true;
            }
            else
            {
                position = -1;
                return false;
            }
        }

        public void Reset()
        {
            position = -1;
        }

    }
}
